import { PlatformAccessory, RemoteController, Service } from "homebridge";
import HarmonyDataProvider from "../DataProviders/HarmonyDataProvider";
import { IActivity } from "../Models/Config";
import { Platform } from "../platform";
import callbackify from "../Util/Callbackify";

/**
 * Enum describing remote key presses from homebridge.
 */
export enum RemoteKey {
  REWIND = 0,
  FAST_FORWARD = 1,
  NEXT_TRACK = 2,
  PREVIOUS_TRACK = 3,
  ARROW_UP = 4,
  ARROW_DOWN = 5,
  ARROW_LEFT = 6,
  ARROW_RIGHT = 7,
  SELECT = 8,
  BACK = 9,
  EXIT = 10,
  PLAY_PAUSE = 11,
  INFORMATION = 15,
}

export class ControlUnit {
  constructor(
    private readonly _platform: Platform,
    private readonly _accessory: PlatformAccessory,
    private _dataProvider: HarmonyDataProvider,
    private _activities: Array<IActivity>
  ) {
    this._accessory
      .getService(this._platform.Service.AccessoryInformation)!
      .setCharacteristic(
        this._platform.Characteristic.Manufacturer,
        "Brandon Watson"
      )
      .setCharacteristic(
        this._platform.Characteristic.Model,
        "Matrix Output Television"
      )
      .setCharacteristic(
        this._platform.Characteristic.SerialNumber,
        "123-456-789"
      );

    const televisionUUID = this._platform.api.hap.uuid.generate(
      `${this._accessory.displayName} Television`
    );
    //Init services
    const televisionService =
      this._accessory.getService(this._platform.Service.Television) ||
      this._accessory.addService(
        this._platform.Service.Television,
        "Television",
        televisionUUID
      );
    this.configureTvService(televisionService);

    const televisionSpeakerUUID = this._platform.api.hap.uuid.generate(
      `${this._accessory.displayName} Television Speaker`
    );
    const televisionSpeakerService =
      this._accessory.getService(this._platform.Service.TelevisionSpeaker) ||
      this._accessory.addService(
        this._platform.Service.TelevisionSpeaker,
        "Television Speaker",
        televisionSpeakerUUID
      );
    this.configureTvSpeakerService(televisionService, televisionSpeakerService);

    this.configureInputSourceService(televisionService);
  }

  /*************
   *
   * Tv Service
   *
   *************/

  /**
   * Configure television service
   */
  private configureTvService(televisionService: Service): void {
    televisionService.setCharacteristic(
      this._platform.Characteristic.ConfiguredName,
      this._accessory.displayName
    );
    //@ts-ignore
    televisionService.setCharacteristic(
      this._platform.Characteristic.SleepDiscoveryMode,
      this._platform.Characteristic.SleepDiscoveryMode.ALWAYS_DISCOVERABLE
    );
    televisionService.setCharacteristic(
      this._platform.Characteristic.ActiveIdentifier,
      1
    );
    televisionService.setCharacteristic(
      this._platform.Characteristic.Active,
      false
    );

    //setup listeners
    televisionService
      .getCharacteristic(this._platform.Characteristic.Active)
      //@ts-ignore
      .on("set", callbackify(this.onSetAccessoryActive))
      //@ts-ignore
      .on("get", callbackify(this.onGetAccessoryActive));

    //Set remote characteristics if is external
    televisionService
      .getCharacteristic(this._platform.Characteristic.RemoteKey)
      //@ts-ignore
      .on("set", callbackify(this.onSetRemoteKey));

    televisionService
      .getCharacteristic(this._platform.Characteristic.ActiveIdentifier)
      //@ts-ignore
      .on("set", callbackify(this.onSetActiveIdentifier))
      //@ts-ignore
      .on("get", callbackify(this.onGetActiveIdentifier));
  }

  /**
   * Event handler for SET active characteristic
   */
  private onSetAccessoryActive = async (value: any) => {
    switch (value) {
      case 0:
        this._dataProvider.powerOff(this._accessory.displayName);
        break;
      //Turn on with first activity
      case 1:
        this._dataProvider.powerOn(
          this._accessory.displayName,
          this._activities[0]
        );
        break;
    }
  };

  /**
   * Event handler for GET active characteristic
   */
  private onGetAccessoryActive = async () => {
    //@ts-ignore
    return this._dataProvider.getIsActive(this._accessory.displayName)
      ? this._platform.Characteristic.Active.ACTIVE
      : this._platform.Characteristic.Active.INACTIVE;
  };

  /**
   * Event handler for SET remote key
   */
  private onSetRemoteKey = async (key: any) => {
    this._dataProvider.sendKeyPress(this._accessory.displayName, key);
  };

  /**
   * Event handler for SET active identifier characteristic
   */
  private onSetActiveIdentifier = async (identifier: any) => {
    this._dataProvider.startActivity(
      this._accessory.displayName,
      this._activities[identifier]
    );
  };

  /**
   * Event handler for GET active identifier characteristic
   */
  private onGetActiveIdentifier = async () => {
    let currentActivity: IActivity = this._dataProvider.getIsActive(
      this._accessory.displayName
    )!;
    let identifier: number = 0;
    if (currentActivity) {
      identifier = this._activities.findIndex(
        (e) => e.DisplayName === currentActivity.DisplayName
      );
    }
    return identifier;
  };

  /******************
   *
   * Speaker Service
   *
   *****************/

  /**
   * Configure Speaker Service
   */
  private configureTvSpeakerService(
    televisionService: Service,
    televisionSpeakerService: Service
  ): void {
    televisionSpeakerService.setCharacteristic(
      this._platform.Characteristic.Name,
      this._accessory.displayName
    );
    //@ts-ignore
    televisionSpeakerService.setCharacteristic(
      this._platform.Characteristic.Active,
      this._platform.Characteristic.Active.ACTIVE
    );
    //@ts-ignore
    televisionSpeakerService.setCharacteristic(
      this._platform.Characteristic.VolumeControlType,
      this._platform.Characteristic.VolumeControlType.ABSOLUTE
    );
    televisionSpeakerService.subtype = this._accessory.displayName + "Volume";
    if (televisionService) {
      //@ts-ignore
      televisionService.addLinkedService(televisionSpeakerService);
    }

    //Setup listeners
    televisionSpeakerService
      .getCharacteristic(this._platform.Characteristic.VolumeSelector)
      //@ts-ignore
      .on("set", callbackify(this.onSetVolumeSelector));
  }

  /**
   * Event handler for SET volume characteristic
   */
  private onSetVolumeSelector = async (value: any) => {
    switch (value) {
      case 0:
        this._dataProvider.volumeUp(this._accessory.displayName);
        break;
      case 1:
        this._dataProvider.volumeDown(this._accessory.displayName);
        break;
    }
  };

  /*****************
   *
   * Input services
   *
   *****************/

  /**
   * Configure input service
   */
  private configureInputSourceService(televisionService: Service): void {
    let inputs: Array<Service> = [];
    this._activities.forEach((activity: IActivity, index: number) => {
      const inputUUID = this._platform.api.hap.uuid.generate(
        `${activity.DisplayName} Input`
      );
      let inputService =
        this._accessory.getService(activity.DisplayName) ||
        this._accessory.addService(
          this._platform.Service.InputSource,
          activity.DisplayName,
          inputUUID
        );

      inputService
        .setCharacteristic(this._platform.Characteristic.Identifier, index)
        .setCharacteristic(
          this._platform.Characteristic.ConfiguredName,
          activity.DisplayName
        )
        .setCharacteristic(
          this._platform.Characteristic.IsConfigured,
          //@ts-ignore
          this._platform.Characteristic.IsConfigured.CONFIGURED
        )
        //@ts-ignore
        .setCharacteristic(
          this._platform.Characteristic.InputSourceType,
          this._platform.Characteristic.InputSourceType.HDMI
        );

      //@ts-ignore
      televisionService!.addLinkedService(inputService);
      inputs.push(inputService);
    });
  }
}
